import type { TRPCClientErrorLike, CreateTRPCProxyClient } from '@trpc/client';
import type { AnyRouter } from '@trpc/server';
import { CreateQueryOptions, CreateMutationOptions, CreateInfiniteQueryOptions, InvalidateQueryFilters, FetchQueryOptions, FetchInfiniteQueryOptions, InfiniteData, RefetchQueryFilters, RefetchOptions, ResetOptions, CancelOptions, Updater, SetDataOptions, QueryClient, InvalidateOptions, QueryFilters, ResetQueryFilters, CreateQueryResult, CreateInfiniteQueryResult, CreateMutationResult } from '@tanstack/svelte-query';
import { CreateQueriesResult } from '@tanstack/svelte-query/build/lib/createQueries';
type WithNevers<T, V> = {
    [K in keyof T]: Exclude<T[K], undefined> extends V ? never : T[K] extends Record<string, unknown> ? Without<T[K], V> : T[K];
};
type Without<T, V, I = WithNevers<T, V>> = Pick<I, {
    [K in keyof I]: I[K] extends never ? never : K;
}[keyof I]>;
type HasQuery = {
    query: (...args: any[]) => any;
};
type HasMutate = {
    mutate: (...args: any[]) => any;
};
type HasSubscribe = {
    subscribe: (...args: any[]) => any;
};
type OnlyQueries<TClient> = Without<TClient, HasMutate | HasSubscribe>;
type QueryType = 'query' | 'infinite' | 'any';
type QueryKey = [
    string[],
    {
        input?: unknown;
        type?: Exclude<QueryType, 'any'>;
    }?
];
type GetQueryKey<TInput = undefined> = TInput extends undefined ? {
    [ProcedureNames.queryKey]: () => QueryKey;
} : {
    /**
     * Method to extract the query key for a procedure
     * @param type - defaults to `any`
     */
    [ProcedureNames.queryKey]: (input: TInput, type?: QueryType) => QueryKey;
} & {};
declare const ContextProcedureNames: {
    readonly client: "client";
    readonly fetch: "fetch";
    readonly prefetch: "prefetch";
    readonly fetchInfinite: "fetchInfinite";
    readonly prefetchInfinite: "prefetchInfinite";
    readonly invalidate: "invalidate";
    readonly refetch: "refetch";
    readonly reset: "reset";
    readonly cancel: "cancel";
    readonly setData: "setData";
    readonly getData: "getData";
    readonly setInfiniteData: "setInfiniteData";
    readonly getInfiniteData: "getInfiniteData";
};
type ContextProcedures<TInput = undefined, TOutput = undefined, TError = undefined> = {
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientfetchquery
     */
    [ContextProcedureNames.fetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<TOutput>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientfetchinfinitequery
     */
    [ContextProcedureNames.fetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<InfiniteData<TOutput>>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientprefetchquery
     */
    [ContextProcedureNames.prefetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientprefetchinfinitequery
     */
    [ContextProcedureNames.prefetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientinvalidatequeries
     */
    [ContextProcedureNames.invalidate](input?: TInput, filters?: InvalidateQueryFilters, options?: InvalidateOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientrefetchqueries
     */
    [ContextProcedureNames.refetch](input?: TInput, filters?: RefetchQueryFilters, options?: RefetchOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientcancelqueries
     */
    [ContextProcedureNames.cancel](input?: TInput, filters?: QueryFilters, options?: CancelOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientresetqueries
     */
    [ContextProcedureNames.reset](input?: TInput, filters?: ResetQueryFilters, options?: ResetOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientsetquerydata
     */
    [ContextProcedureNames.setData](input: TInput, updater: Updater<TOutput | undefined, TOutput | undefined>, options?: SetDataOptions): void;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientsetquerydata
     */
    [ContextProcedureNames.setInfiniteData](input: TInput, updater: Updater<InfiniteData<TOutput> | undefined, InfiniteData<TOutput> | undefined>, options?: SetDataOptions): void;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientgetquerydata
     */
    [ContextProcedureNames.getData](input?: TInput, filters?: QueryFilters): TOutput | undefined;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientgetquerydata
     */
    [ContextProcedureNames.getInfiniteData](input?: TInput, filters?: QueryFilters): InfiniteData<TOutput> | undefined;
};
type AddContextPropTypes<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? ContextProcedures<Parameters<TClient[K]['query']>[0], Awaited<ReturnType<TClient[K]['query']>>, TError> : AddContextPropTypes<TClient[K], TError> & Pick<ContextProcedures, typeof ContextProcedureNames.invalidate>;
};
type CreateContext<TClient, TError> = AddContextPropTypes<OnlyQueries<TClient>, TError> & Pick<ContextProcedures, typeof ContextProcedureNames.invalidate> & {
    [ContextProcedureNames.client]: TClient;
};
type CreateQueryOptionsForCreateQueries<TInput, TError> = Omit<CreateQueryOptions<TInput, TError>, 'context'>;
type CreateQueriesRecord<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? (input: Parameters<TClient[K]['query']>[0], opts?: CreateQueryOptionsForCreateQueries<Awaited<ReturnType<TClient[K]['query']>>, TError>) => CreateQueryOptionsForCreateQueries<Awaited<ReturnType<TClient[K]['query']>>, TError> : CreateQueriesRecord<TClient[K], TError>;
};
type CreateQueries<TClient, TError> = <TOpts extends CreateQueryOptionsForCreateQueries<any, any>[]>(queriesCallback: (t: CreateQueriesRecord<OnlyQueries<TClient>, TError>) => readonly [...TOpts]) => CreateQueriesResult<TOpts>;
type CreateServerQueries<TClient, TError> = <TOpts extends CreateQueryOptionsForCreateQueries<any, any>[]>(queriesCallback: (t: CreateQueriesRecord<OnlyQueries<TClient>, TError>) => readonly [...TOpts]) => Promise<() => CreateQueriesResult<TOpts>>;
declare const ProcedureNames: {
    readonly query: "createQuery";
    readonly serverQuery: "createServerQuery";
    readonly infiniteQuery: "createInfiniteQuery";
    readonly serverInfiniteQuery: "createServerInfiniteQuery";
    readonly mutate: "createMutation";
    readonly subscribe: "createSubscription";
    readonly queryKey: "getQueryKey";
    readonly context: "createContext";
    readonly queries: "createQueries";
    readonly serverQueries: "createServerQueries";
};
type CreateQueryProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.query]: (input: TInput, opts?: CreateQueryOptions<TOutput, TError>) => CreateQueryResult<TOutput, TError>;
    [ProcedureNames.serverQuery]: (input: TInput, opts?: CreateQueryOptions<TOutput, TError>) => Promise<() => CreateQueryResult<TOutput, TError>>;
} & {};
type CreateInfiniteQueryProcedure<TInput, TOutput, TError> = (TInput extends {
    cursor?: any;
} ? {
    [ProcedureNames.infiniteQuery]: (input: Omit<TInput, 'cursor'>, opts?: CreateInfiniteQueryOptions<TOutput, TError>) => CreateInfiniteQueryResult<TOutput, TError>;
    [ProcedureNames.serverInfiniteQuery]: (input: Omit<TInput, 'cursor'>, opts?: CreateInfiniteQueryOptions<TOutput, TError>) => Promise<() => CreateInfiniteQueryResult<TOutput, TError>>;
} : {}) & {};
type QueryProcedures<TInput, TOutput, TError> = GetQueryKey<TInput> & CreateQueryProcedure<TInput, TOutput, TError> & CreateInfiniteQueryProcedure<TInput, TOutput, TError>;
type CreateMutationProcedure<TInput, TOutput, TError, TContext = unknown> = {
    [ProcedureNames.mutate]: (opts?: CreateMutationOptions<TOutput, TError, TInput, TContext>) => CreateMutationResult<TOutput, TError, TInput, TContext>;
} & {};
type CreateTRPCSubscriptionOptions<TOutput, TError> = {
    enabled?: boolean;
    onStarted?: () => void;
    onData: (data: TOutput) => void;
    onError?: (err: TError) => void;
};
type GetSubscriptionOutput<TOpts> = TOpts extends unknown & Partial<infer A> ? A extends {
    onData: any;
} ? Parameters<A['onData']>[0] : never : never;
type CreateSubscriptionProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.subscribe]: (input: TInput, opts?: CreateTRPCSubscriptionOptions<TOutput, TError>) => void;
} & {};
type AddQueryPropTypes<TClient, TError> = TClient extends Record<any, any> ? {
    [K in keyof TClient]: TClient[K] extends HasQuery ? QueryProcedures<Parameters<TClient[K]['query']>[0], Awaited<ReturnType<TClient[K]['query']>>, TError> & {} : TClient[K] extends HasMutate ? CreateMutationProcedure<Parameters<TClient[K]['mutate']>[0], Awaited<ReturnType<TClient[K]['mutate']>>, TError> : TClient[K] extends HasSubscribe ? CreateSubscriptionProcedure<Parameters<TClient[K]['subscribe']>[0], GetSubscriptionOutput<Parameters<TClient[K]['subscribe']>[1]>, TError> : GetQueryKey & AddQueryPropTypes<TClient[K], TError>;
} : TClient;
export declare function svelteQueryWrapper<TRouter extends AnyRouter>({ client, queryClient, }: {
    client: CreateTRPCProxyClient<TRouter>;
    queryClient?: QueryClient;
}): AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> & (AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> extends Record<any, any> ? {
    createContext(): CreateContext<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    createQueries: CreateQueries<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    createServerQueries: CreateServerQueries<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
} : {});
export {};
